import java.awt.*;
import java.awt.event.*;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.UnknownHostException;

import javax.swing.*;
import javax.swing.table.*;

/**
 * This class will display a list of servers and their status.
 *
 * The 'ServerStatus.txt' file layout is as follows:
 *
 * '#' or ';' at the beginning of a line means the line is a comment line
 * 192.168.10.1           <- is a single IP address to monitor
 * 192.168.10.[100-150]   <- is a range of IP addresses to monitor
 * xyzserver001           <- hostname of a remote server
 * abc001.acme.com        <- fully-qualified domain name
 *
 * Note: The range value is only allowed for the 4th number of the IP address.
 *
 * Add at least 1 space after the ip address / hostname to have a
 * description included in the window.
 *
 * This class can ONLY be used with Java v1.5 or higher.
 *
 * @author Roger Lacroix, Capitalware Inc.
 * @version 1.0.0
 */
public class ServerStatus extends JFrame implements ComponentListener
{
   public static final String    VERSION = "1.0.1";
   public static final String    RELEASE_DATE = "2014/06/16";
   public static final String    COPYRIGHT_DATE = "2008-2014";

   public static final String    ONLINE  = "Online";
   public static final String    OFFLINE = "Offline";

   private static final int      MIN_WIDTH  = 500;
   private static final int      MIN_HEIGHT = 320;
   private static final int      WAIT_TIME  = 60*1000; /* 1 minute */

   private static final String   FILE_NAME = "ServerStatus.txt";

   private boolean               stopMonitoring = false;

   private JLabel                statusLine;

   private ServerStatusModel     modelData;
   private JTable                table;
   private Timer                 timer;

   /**
    * Mainline
    *
    * @param argv
    */
   public static void main(String argv[])
   {
      new ServerStatus();
   }

   /**
    * The constructor
    */
   public ServerStatus()
   {
      super();
      addComponentListener(this);

      setTitle("ServerStatus v" + VERSION);
      setSize(MIN_WIDTH, MIN_HEIGHT);
      setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

      getContentPane().setLayout(new GridBagLayout());

      addWindowListener(new WindowAdapter()
      {
         public void windowClosing(WindowEvent we)
         {
            dismiss();
         }
      });

      setLocationRelativeTo(this);
      setVisible(true);

      readFile();
      setupMainWindow();

      /**
       * Immediately get the status.
       * Don't want to wait for the timer to pop.
       */
      IPListRunner iplr = new IPListRunner();
      iplr.start();

      /**
       * Setup a 1 minute timer to get the status
       */
      startTimer();
   }

   /**
    * Add a component to main pane.
    *
    * @param c
    * @param gbc
    * @param x
    * @param y
    * @param w
    * @param h
    */
   private void add(Component c, GridBagConstraints gbc2, int x, int y, int w, int h)
   {
      gbc2.gridx = x;
      gbc2.gridy = y;
      gbc2.gridwidth = w;
      gbc2.gridheight = h;
      getContentPane().add(c, gbc2);
   }

   /**
    * Size out the columns of the JTable.
    */
   private void sizeOutColumns()
   {
      TableColumnModel columnModel = table.getColumnModel();
      RenderRedGreen rrg = new RenderRedGreen();
      DefaultTableCellRenderer dtcr1 = new DefaultTableCellRenderer()
      {
         public void setValue(Object value)
         {
            super.setValue(value);
         }
      };
      DefaultTableCellRenderer dtcr2 = new DefaultTableCellRenderer()
      {
         public void setValue(Object value)
         {
            super.setValue(value);
         }
      };

      /*
       * Start column resizing
       */
      TableColumn column = columnModel.getColumn(0);
      column.setPreferredWidth(60);
      column.setMaxWidth(60);
      column.setCellRenderer(rrg);

      /* next column */
      column = columnModel.getColumn(1);
      column.setPreferredWidth(175);
      column.setMaxWidth(175);
      dtcr1.setHorizontalAlignment(DefaultTableCellRenderer.CENTER);
      column.setCellRenderer(dtcr1);

      /* next column */
      column = columnModel.getColumn(2);
      dtcr2.setHorizontalAlignment(DefaultTableCellRenderer.LEFT);
      column.setCellRenderer(dtcr2);
   }

   /**
    * Build the table with the list of queues.
    */
   private void setupMainWindow()
   {
      table = new JTable(modelData);
      table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

      ((DefaultTableCellRenderer) table.getTableHeader().getDefaultRenderer()).setHorizontalAlignment(SwingConstants.CENTER);

      // Not allow the User to Move a Column in a JTable Component
      table.getTableHeader().setReorderingAllowed(false);

      GridBagConstraints gbc = new GridBagConstraints();
      gbc.anchor = GridBagConstraints.CENTER;
      gbc.fill = GridBagConstraints.BOTH;
      gbc.weightx = 0.9;
      gbc.weighty = 1.0;
      gbc.insets = new Insets(0, 0, 0, 0); // Top, Left, Bottom, Right
      JScrollPane JSp1 = new JScrollPane(table);
      add(JSp1, gbc, 0, 0, 0, 100); // c , r, cw, rh

      sizeOutColumns();

      /**
       * Add a status line at the bottom of the screen
       */
      gbc.anchor = GridBagConstraints.WEST;
      gbc.fill = GridBagConstraints.HORIZONTAL;
      gbc.weightx = 0.1;
      gbc.weighty = 0;

      statusLine = new JLabel(" ");
      statusLine.setFont(new Font("Arial", Font.PLAIN, 12));
      statusLine.setForeground(Color.black);
      statusLine.setBorder(BorderFactory.createLoweredBevelBorder());
      add(statusLine, gbc, 0, 101, 0, 1); // c , r, cw, rh

      // force the pane to be updated.
      this.setVisible(true);
      repaint();
   }

   /**
    * Read the file with the IP addresses. One IP address per line.
    */
   private void readFile()
   {
      ProcessInputFile pif = new ProcessInputFile(this);
      try
      {
         pif.readFile(FILE_NAME);
      }
      catch (FileNotFoundException e)
      {
         JOptionPane.showMessageDialog(this,
                                       "File not found: "+FILE_NAME,
                                       "File Error",
                                       JOptionPane.ERROR_MESSAGE);
         System.exit(0);  // Cannot continue without a file.
      }

      modelData = new ServerStatusModel(pif.getRowObjects());
   }

   /**
    * Start a timer thread to loop through the IP addresses once a minute.
    */
   private void startTimer()
   {
      timer = new Timer(WAIT_TIME, new ActionListener()
      {
         public void actionPerformed(ActionEvent ae)
         {
            IPListRunner iplr = new IPListRunner();
            iplr.start();
         }
      });
      timer.start();
   }

   /**
    * Time to cleanup the JFrame
    */
   void dismiss()
   {
      stopMonitoring = true;

      this.setVisible(false);
      this.dispose();
   }

   /**
    * Handle the user resizing the window
    * @param e
    */
   public void componentResized(ComponentEvent e)
   {
      int width = getWidth();
      int height = getHeight();

      if ( (width < MIN_WIDTH) || (height < MIN_HEIGHT) )
      {
         if (width < MIN_WIDTH)
            width = MIN_WIDTH;

         if (height < MIN_HEIGHT)
            height = MIN_HEIGHT;

         setSize(width, height);
      }
   }

   public void componentMoved(ComponentEvent e)
   {}

   public void componentShown(ComponentEvent e)
   {}

   public void componentHidden(ComponentEvent e)
   {}
   
   /**
    * Need a separate thread to handle the 'pinging' (isReachable) the remote server.
    * This is because isReachable will timeout for 2 seconds waiting on the remote
    * server and if it is not in a separate thread it will lock the GUI.
    *
    * There also seems to be some issue with using isReachable in many threads
    * at the same time.  Hence, do each isReachable serially.
    */
   private class IPListRunner extends Thread
   {
      public void run()
      {
         boolean isWindows = System.getProperty("os.name").toLowerCase().contains("window");
         int badCounter = 0;
         boolean flag = false;

         for (int i = 0; i < modelData.getRowCount(); i++)
         {
            if (stopMonitoring)
            {
               if (timer != null)
                  timer.stop();
               return;
            }

            try
            {
               /**
                * Ping the ip address or hostname or domain name.
                */
               flag = false;  // assume it is offline
               statusLine.setText(" Pinging "+(String) modelData.getValueAt(i, 1));

               flag = doCommand((String) modelData.getValueAt(i, 1), isWindows);
               
            }
            catch (UnknownHostException e)
            {
               e.printStackTrace();
            }
            catch (IOException e)
            {
               e.printStackTrace();
            }
            catch (Exception e)
            {
               e.printStackTrace();
            }

            statusLine.setText(" "); /* blank out the status line */

            /**
             * Post the results.
             */
            modelData.setValueAt(i, 0, (flag ? ONLINE : OFFLINE));

            /**
             * Can only handle a maximum of 29 bad results because we wait 2 seconds * 29
             * plus 2 second timeout means 60 seconds and the timer fires every 60 seconds.
             * We should have only 1 outstanding timer!!!
             */
            if (!flag)
            {
               if (++badCounter >= 29)
                  return;
            }
         }
      }
      
      /**
       * Issue the OS ping command
       * @param ipAddress
       * @param isWindows
       * @return flag
       * @throws IOException
       */
      public boolean doCommand(String ipAddress, boolean isWindows)  throws IOException
      {
         String s = null;
         boolean flag = true;

         ProcessBuilder pb = new ProcessBuilder("ping",isWindows ? "-n" : "-c", "1", ipAddress);
//         ProcessBuilder pb = new ProcessBuilder("ping",isWindows ? "-n" : "-c", "1", "-w", "1000", ipAddress);

         Process process = pb.start();

         BufferedReader stdInput = new BufferedReader(new InputStreamReader(process.getInputStream()));
//         BufferedReader stdError = new BufferedReader(new InputStreamReader(process.getErrorStream()));

         while ((s = stdInput.readLine()) != null)
         {
//            System.out.println(s);
            if (s.contains("Destination host unreachable"))
            {
               flag =  false;
               break;
            }
            else if (s.contains("Request timed out"))
            {
               flag =  false;
               break;
            }
         }
         
         return flag;
      }
   }
}
